// Copyright 2004, 2005 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry;


/**
 *  Interface that defines classes that may be messaged by the direct
 *  service.
 */
public interface IDirect extends IComponent, IDynamicInvoker
{
    /**
     *  Invoked by the direct service to have the component perform
     *  the appropriate action.  The {@link org.apache.tapestry.link.DirectLink} component will
     *  notify its listener.
     *
     * @param cycle
     *          The current request.
     */
    void trigger(IRequestCycle cycle);

    /**
     *  Invoked by the direct service to query the component as to
     *  whether it is stateful.  If stateful and no 
     *  {@link javax.servlet.http.HttpSession} is active, then a
     *  {@link org.apache.tapestry.StaleSessionException} is
     *  thrown by the service.
     *
     * @return Whether or not invocation should include state information and validate
     *          that a valid session exists when triggered.
     * @since 2.3
     */
    boolean isStateful();
}
